
/* module.c: Sony CY8CMBR31xx Driver.
 *
 * Copyright 2016 Sony Corporation.
 * Author: Sony Corporation.
 *
 * This package is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/i2c.h>
#include <linux/miscdevice.h>
#include <linux/input.h>
#include <linux/dma-mapping.h>
#include <asm/uaccess.h>
#include <mach/mt_gpio.h>
#include <mach/eint.h>
#include <icx_key.h>
#include "cy8cmbr31xx.h"
#include "regs.h"

#define DRV_NAME					"cy8cmbr31xx"

#define DBGTRACE(fmt, ...)			//
//#define DBGTRACE(fmt, ...)			printk("\e[32m[" DRV_NAME "] "       fmt "\e[0m\n", ## __VA_ARGS__)
#define ERRTRACE(fmt, ...)			printk("\e[31m[" DRV_NAME "] error " fmt "\e[0m\n", ## __VA_ARGS__)

//#define DEBUG_CONFIG				/* デバッグ用のコンフィグ使用           */
//#define SHOW_CALC_CRC				/* CRC の計算結果を表示（データ作成用） */


	enum {
		TI2CBOOT					=	15,							/* 起動時間                       */
		TBOOT_SYS_WF				=	1350,						/* システム診断・EMC有効起動時間  */
		TXRES						=	5,							/* 外部リセットパルス幅           */
		TI2C_LATENCY_MAX			=	50,							/* I2C コマンドから実行までの時間 */
		TI2C_LATENCY_SAVE			=	220,						/* SAVE_CHECK_CRC の時間          */
		I2C_SPEED					=	400,						/* I2C 通信速度                   */
//		I2C_RETRY					=	20,							/* I2C リトライ回数               */
		I2C_RETRY					=	10,							/* I2C リトライ回数               */
		BOOT_INTERVAL				=	20,							/* 起動待ち間隔                   */
		BUTTON_MAX					=	16							/* ボタン最大数                   */
	} ;

	/* 種類 */
	enum {
		TYPE_CY8CMBR3102,
		TYPE_CY8CMBR3106S,
		TYPE_CY8CMBR3108,
		TYPE_CY8CMBR3110,
		TYPE_CY8CMBR3116,
		TYPE_MAX
	} ;

	/* プライベートデータ */
	struct priv {
		struct i2c_client			*p_client ;						/* I2C クライアント       */
		struct miscdevice			misc ;							/* MISC デバイス          */
		struct input_dev			*p_input ;						/* 入力デバイス           */
		struct workqueue_struct		*p_workqueue ;					/* ワークキュー           */
		struct work_struct			work ;							/* ワーク                 */
		struct timer_list			timer ;							/* タイマ                 */
		struct cy8cmbr31xx_config	*p_config ;						/* 構成                   */
		u8							type ;							/* 種類                   */
		u8							init ;							/* 初期化完了フラグ       */
		u16							mask ;							/* 状態マスク             */
		u16							state ;							/* 以前の状態             */
		u32							polling ;						/* ポーリング時間         */
	} ;


static struct priv					*m_p_priv ;


//# 初期値
//static const u8						cy8cmbr3108_resume[]={
//										0x0f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x80,0x80,0x80,
//										0x80,0x80,0x80,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,0x0c,0x00,0x32,
//										0x33,0x33,0x00,0x00,0x00,0x00,0x00,0x80,0x05,0x00,0x00,0x02,0x00,0x02,0x00,0x00,
//										0x05,0x00,0x32,0x14,0x14,0x1e,0x1e,0x00,0x00,0x1e,0x1e,0x00,0x00,0x00,0x01,0x01,
//										0x00,0x0f,0x0f,0x0f,0x0f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x54,0x03,0x01,0x08,
//										0x00,0x37,0x06,0x00,0x00,0x0a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
//										0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
//										0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x2c,0xb1
//									} ;


#ifdef  DEBUG_CONFIG
static const u8							cy8cmbr3108_regs[]={
											0x70,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x80,0x80,0x80,
											0x80,0x80,0x80,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,0x0c,0x00,0x32,
											0x33,0x33,0x00,0x00,0x00,0x00,0x00,0x80,0x05,0x00,0x00,0x02,0x00,0x02,0x00,0x00,
											0x05,0x00,0x32,0x14,0x14,0x1e,0x1e,0x00,0x00,0x1e,0x1e,0x00,0x00,0x00,0x01,0x01,
											0x00,0x0f,0x0f,0x0f,0x0f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,0x01,0x08,
											0x00,0x37,0x06,0x00,0x00,0x0a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
											0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
											0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x45,0x24
										} ;

static const struct cy8cmbr31xx_keyinfo	cy8cmbr3108_keyinfo[]={
											{ 1<<0, KEY_1 },
											{ 1<<1, KEY_2 },

											{ 1<<4, KEY_VOLUMEDOWN },
											{ 1<<5, KEY_VOLUMEUP   },
											{ 1<<6, KEY_LEFT       }
										} ;

static struct cy8cmbr31xx_config		cy8cmbr3108_config={
											&cy8cmbr3108_regs[0],
											-1,
											-1,
											ARRAY_SIZE( cy8cmbr3108_keyinfo ),
											&cy8cmbr3108_keyinfo[0]
										} ;
#endif/*DEBUG_CONFIG*/

/*
 * sleep & reschedule kernel task
 *
 *  IN : jiffies        = time to wait (micro sec)
 *  OUT: none
 */
static inline void      my_msleep(long msec)
{
	long	wait;

	wait = (msec * HZ + (1000 - 1)) / 1000;
	if (wait <= 0)
		wait = 1L;
	set_current_state(TASK_UNINTERRUPTIBLE);
	schedule_timeout(wait);
}


/* -------------------------------------------------------------------------------- */
/*   レジスタを読み込む                                                             */
/*                                                                                  */
/*   IN : p_priv  = プライベートデータ                                              */
/*        address = レジスタアドレス                                                */
/*        p_data  = データ格納アドレス                                              */
/*        size    = データサイズ                                                    */
/*   OUT: 0                                                                         */
/*        -EIO                                                                      */
/* -------------------------------------------------------------------------------- */
static int read_regs(struct priv *p_priv, u8 address, void *p_data, int size)
{
	struct i2c_msg	msg[2] ;
	int				result ;
	uint			index ;


	memset(&msg[0], 0, sizeof( msg )) ;
	msg[0].addr   = p_priv->p_client->addr ;
	msg[0].flags  = 0 ;
	msg[0].len    = 1 ;
	msg[0].buf    = &address ;
	msg[0].timing = I2C_SPEED ;
	msg[1].addr   = msg[0].addr ;
	msg[1].flags  = I2C_M_RD ;
	msg[1].len    = size ;
	msg[1].buf    = p_data ;
	msg[1].timing = I2C_SPEED ;

	for(index=0; index<I2C_RETRY; index++) {
		result = i2c_transfer(p_priv->p_client->adapter, &msg[0], ARRAY_SIZE( msg )) ;
		if( result == ARRAY_SIZE( msg ) ) {
			return 0 ;
		}
		my_msleep(10);
	}

	return -EIO ;
}


/* -------------------------------------------------------------------------------- */
/*   8bit レジスタを読み込む                                                        */
/*                                                                                  */
/*   IN : p_priv  = プライベートデータ                                              */
/*        address = レジスタアドレス                                                */
/*        p_data  = データ格納アドレス                                              */
/*   OUT: 0                                                                         */
/*        -EIO                                                                      */
/* -------------------------------------------------------------------------------- */
static inline int read_reg8(struct priv *p_priv, u8 address, u8 *p_data)
{
	u8		data ;
	int		result ;


	result = read_regs(p_priv, address, &data, sizeof( data )) ;
	if( result ) {
		return result ;
	}

	*p_data = data ;

	return 0 ;
}


/* -------------------------------------------------------------------------------- */
/*   16bit レジスタを読み込む                                                       */
/*                                                                                  */
/*   IN : p_priv  = プライベートデータ                                              */
/*        address = レジスタアドレス                                                */
/*        p_data  = データ格納アドレス                                              */
/*   OUT: 0                                                                         */
/*        -EIO                                                                      */
/* -------------------------------------------------------------------------------- */
static inline int read_reg16(struct priv *p_priv, u8 address, u16 *p_data)
{
	u8		data[2] ;
	int		result ;


	result = read_regs(p_priv, address, &data[0], sizeof( data )) ;
	if( result ) {
		return -EIO ;
	}

	*p_data = ((u16 )data[1] << 8) | data[0] ;

	return 0 ;
}


/* -------------------------------------------------------------------------------- */
/*   レジスタへ値を書き込む                                                         */
/*                                                                                  */
/*   IN : p_priv  = プライベートデータ                                              */
/*        address = レジスタアドレス                                                */
/*        p_data  = データアドレス                                                  */
/*        size    = データサイズ                                                    */
/*        wait    = コマンド待ち時間（msec）                                        */
/*   OUT: 0                                                                         */
/*        -EIO                                                                      */
/*        -ENOMEM                                                                   */
/* -------------------------------------------------------------------------------- */
static int write_regs(struct priv *p_priv, u8 address, const void *p_data, int size, int wait)
{
	struct i2c_msg	msg ;
	dma_addr_t		phy_addr ;
	u8				buff[8], *p_buff ;
	int				result ;
	uint			index ;

	/* 初期化 */
	memset(&msg, 0, sizeof( msg )) ;
	msg.addr   = p_priv->p_client->addr ;
	msg.flags  = 0 ;
	msg.len    = 1 + size ;
	msg.buf    = &buff[0] ;
	msg.timing = I2C_SPEED ;
	p_buff     = &buff[0] ;

	/* DMA バッファ確保 */
	if( msg.len > 8 ) {
		p_buff = dma_alloc_coherent(0, msg.len, &phy_addr, GFP_KERNEL) ;
		if( p_buff == NULL ) {
			ERRTRACE("%s(%d)", __func__, __LINE__) ;
			return -ENOMEM ;
		}
		msg.buf      = (unsigned char *)phy_addr ;
		msg.ext_flag = I2C_DMA_FLAG ;
	}

	/* データコピー */
	p_buff[0] = address ;
	memcpy(&p_buff[1], p_data, size) ;

	/* 転送 */
	for(index=0; index<I2C_RETRY; index++) {
		result = i2c_transfer(p_priv->p_client->adapter, &msg, 1) ;
		if( result == 1 ) {
			break ;
		}
		my_msleep(10);
	}

	/* DMA バッファ解放 */
	if( p_buff != &buff[0] ) {
		dma_free_coherent(0, 1+size, &p_buff[0], phy_addr) ;
	}

	if( result != 1 ) {
		if( result >= 0 ) {
			result = -EIO ;
		}
		return result ;
	}

	/* コマンド実行待ち */
	if( wait ) {
		msleep( wait ) ;
	}

	return 0 ;
}


/* -------------------------------------------------------------------------------- */
/*   8bit レジスタへ値を書き込む                                                    */
/*                                                                                  */
/*   IN : p_priv  = プライベートデータ                                              */
/*        address = レジスタアドレス                                                */
/*        p_data  = データアドレス                                                  */
/*        size    = データサイズ                                                    */
/*        wait    = コマンド待ち時間（msec）                                        */
/*   OUT: 0                                                                         */
/*        -EIO                                                                      */
/* -------------------------------------------------------------------------------- */
static inline int write_reg8(struct priv *p_priv, u8 address, u8 value, int wait)
{
	return write_regs(p_priv, address, &value, 1, wait) ;
}


/* -------------------------------------------------------------------------------- */
/*   16bit レジスタへ値を書き込む                                                   */
/*                                                                                  */
/*   IN : p_priv  = プライベートデータ                                              */
/*        address = レジスタアドレス                                                */
/*        p_data  = データアドレス                                                  */
/*        size    = データサイズ                                                    */
/*        wait    = コマンド待ち時間（msec）                                        */
/*   OUT: 0                                                                         */
/*        -EIO                                                                      */
/* -------------------------------------------------------------------------------- */
static inline int write_reg16(struct priv *p_priv, u8 address, u16 value, int wait)
{
#ifdef  __LITTLE_ENDIAN
	return write_regs(p_priv, address, &value, 2, wait) ;
#else
	u8	data[2] ;

	data[0] = (u8 )(value >> 0) ;
	data[1] = (u8 )(value >> 8) ;
	return write_regs(p_priv, address, &data[0], 2, wait) ;
#endif/*__LITTLE_ENDIAN__*/
}


/* -------------------------------------------------------------------------------- */
/*   リセット                                                                       */
/*                                                                                  */
/*   IN : p_priv = プライベートデータ                                               */
/*   OUT: 0                                                                         */
/*        -EIO                                                                      */
/* -------------------------------------------------------------------------------- */
static int reset( struct priv *p_priv )
{
	struct cy8cmbr31xx_config	*p_config=p_priv->p_config ;
	int							result ;


	if( p_config->reset >= 0 ) {	/* ハードリセット */

		DBGTRACE("hard reset") ;

		mt_set_gpio_out(p_config->reset, 0) ;

		msleep( TXRES ) ;

		mt_set_gpio_out(p_config->reset, 1) ;

	} else {						/* ソフトリセット */

		DBGTRACE("soft reset") ;

		msleep( TI2CBOOT ) ;

		result = write_reg8(p_priv, CTRL_CMD, SW_RESET, TI2C_LATENCY_MAX) ;
		if( result ) {
			ERRTRACE("%s(%d) : %d", __func__, __LINE__, result) ;
			return result ;
		}

	}

	msleep( TI2CBOOT ) ;

	return 0 ;
}


/* -------------------------------------------------------------------------------- */
/*   起動待ち                                                                       */
/*                                                                                  */
/*   IN : p_priv = プライベートデータ                                               */
/*   OUT: 0                                                                         */
/*        -EIO                                                                      */
/* -------------------------------------------------------------------------------- */
static int wait_boot( struct priv *p_priv )
{
	static const u16	deviceId[TYPE_MAX]={
							2561,		/* TYPE_CY8CMBR3102  */
							2566,		/* TYPE_CY8CMBR3106S */
							2563,		/* TYPE_CY8CMBR3108  */
							2562,		/* TYPE_CY8CMBR3110  */
							2565,		/* TYPE_CY8CMBR3116  */
						} ;
	u16					value ;
	u32					base ;
	int					result ;


	base = jiffies ;

	while( 1 ) {
		result = read_reg16(p_priv, DEVICE_ID, &value) ;
		if( !result ) {
			if( value == deviceId[p_priv->type] ) {
				break ;
			}
		}

		if( (jiffies-base) > msecs_to_jiffies( TBOOT_SYS_WF ) ) {
			return -EIO ;
		}

		DBGTRACE("%s(%d) : retry", __func__, __LINE__) ;

		msleep( BOOT_INTERVAL ) ;
	}

	return 0 ;
}


/* -------------------------------------------------------------------------------- */
/*   フラッシュプログラム                                                           */
/*                                                                                  */
/*   IN : p_priv   = プライベートデータ                                             */
/*        p_config = コンフィグレーションデータ                                     */
/*   OUT: 0                                                                         */
/*        -EIO                                                                      */
/* -------------------------------------------------------------------------------- */
static int programFlash(struct priv *p_priv, const u8 *p_config)
{
	u16		value ;
	int		result ;

	/* リセット */
	reset( p_priv ) ;

	/* 起動待ち */
	result = wait_boot( p_priv ) ;
	if( result ) {
		ERRTRACE("%s(%d) : %d", __func__, __LINE__, result) ;
		return result ;
	}

	/* コンフィグレーションデータ書き込み */
	result = write_regs(p_priv, DATA_OFFSET, p_config, 128, TI2C_LATENCY_MAX) ;
	if( result ) {
		ERRTRACE("%s(%d) : %d", __func__, __LINE__, result) ;
		return result ;
	}

	/* フラッシュ書き込み */
	result = write_reg8(p_priv, CTRL_CMD, SAVE_CHECK_CRC, TI2C_LATENCY_SAVE) ;
	if( result ) {
		ERRTRACE("%s(%d) : %d", __func__, __LINE__, result) ;
		return result ;
	}

#ifdef  SHOW_CALC_CRC
{
	result = write_reg8(p_priv, CTRL_CMD, CMD_OP_CODE_3, TI2C_LATENCY_MAX) ;
	DBGTRACE("%s(%d) : %d", __func__, __LINE__, result) ;

	result = read_reg16(p_priv, CALC_CRC, &value) ;
	DBGTRACE("%s(%d) : %d, 0x%02x,0x%02x", __func__, __LINE__, result, (u8 )value, (u8 )(value>>8)) ;

	ERRTRACE("%s(%d)", __func__, __LINE__) ;

	return -EIO ;
}
#endif/*SHOW_CALC_CRC*/


	/* 状態取得 */
	result = read_reg16(p_priv, CTRL_CMD_STATUS, &value) ;
	if( result ) {
		ERRTRACE("%s(%d) : %d", __func__, __LINE__, result) ;
		return result ;
	}

	if( value & 0xff01 ) {
		ERRTRACE("%s(%d) : 0x%04x", __func__, __LINE__, value) ;
		return -EIO ;
	}

	/* ソフトリセット */
	result = write_reg8(p_priv, CTRL_CMD, SW_RESET, TI2C_LATENCY_MAX) ;
	if( result ) {
		ERRTRACE("%s(%d) : %d", __func__, __LINE__, result) ;
		return result ;
	}

	return 0 ;
}


/* -------------------------------------------------------------------------------- */
/*   ホスト割り込み処理                                                             */
/*                                                                                  */
/*   IN : なし                                                                      */
/*   OUT: なし                                                                      */
/* -------------------------------------------------------------------------------- */
static void host_isr( void )
{
	struct priv		*p_priv=m_p_priv ;


	queue_work(p_priv->p_workqueue, &p_priv->work) ;
}


/* -------------------------------------------------------------------------------- */
/*   タイマー処理                                                                   */
/*                                                                                  */
/*   IN : data = プライベートデータ                                                 */
/*   OUT: なし                                                                      */
/* -------------------------------------------------------------------------------- */
static void timer_proc( unsigned long data )
{
	struct priv		*p_priv=(struct priv *)data ;


	queue_work(p_priv->p_workqueue, &p_priv->work) ;

	p_priv->timer.expires = jiffies + p_priv->polling ;
	add_timer( &p_priv->timer ) ;
}


/* -------------------------------------------------------------------------------- */
/*   ワークキュー処理                                                               */
/*                                                                                  */
/*   IN : p_wok = プライベートデータ内のワークアドレス                              */
/*   OUT: なし                                                                      */
/* -------------------------------------------------------------------------------- */
static void workqueue_proc( struct work_struct *p_work )
{
	struct priv					*p_priv=container_of(p_work, struct priv, work) ;
	struct cy8cmbr31xx_config	const *p_config=p_priv->p_config ;
	struct cy8cmbr31xx_keyinfo	const *p_key_info ;
	u8							prox ;
	u16							state, update ;
	int							result ;
	uint						index ;


	do {
		/* ボタン状態取得 */
		result = read_reg16(p_priv, BUTTON_STAT, &state) ;
		if( result ) {
			break ;
		}

		if( p_priv->type == TYPE_CY8CMBR3106S ) {
			result = read_reg8(p_priv, PROX_STAT, &prox) ;
			if( result ) {
				break ;
			}
			state = (state & ((1<<6)-1)) | (prox << 6) ;
		}

		state &= p_priv->mask ;

		/* 状態に変化がないなら無視 */
		update = state ^ p_priv->state ;
		if( !update ) {
			break ;
		}
		p_priv->state = state ;

		/* ボタン状態更新 */
		p_key_info = &p_config->p_key_info[0] ;
		for(index=0; index<p_config->count; index++,p_key_info++) {
			if( update & p_key_info->key_bit ) {
				input_event(p_priv->p_input, EV_KEY, p_key_info->key_code, (state&p_key_info->key_bit)!=0) ;

				update &= ~p_key_info->key_bit ;
				if( !update ) {
					break ;
				}
			}
		}

		input_sync( p_priv->p_input ) ;
	} while( 0 ) ;
}


/* -------------------------------------------------------------------------------- */
/*   開く                                                                           */
/*                                                                                  */
/*   IN : p_inode = i ノード                                                        */
/*        p_file  = ファイル                                                        */
/*   OUT: 0                                                                         */
/* -------------------------------------------------------------------------------- */
static int cy8cmbr31xx_open(struct inode *p_inode, struct file *p_file)
{
	p_file->private_data = container_of(p_file->private_data, struct priv, misc) ;

	return 0 ;
}


/* -------------------------------------------------------------------------------- */
/*   ioctl                                                                          */
/*                                                                                  */
/*   IN : p_file = ファイル                                                         */
/*        cmd    = コマンド                                                         */
/*        arg    = 引数                                                             */
/*   OUT: 0                                                                         */
/*        キーコード数                                                              */
/*        -EFAULT                                                                   */
/*        -EINVAL                                                                   */
/* -------------------------------------------------------------------------------- */
static long cy8cmbr31xx_ioctl(struct file *p_file, unsigned int cmd, unsigned long arg)
{
	struct priv					*p_priv=p_file->private_data ;
	struct cy8cmbr31xx_config	const *p_config=p_priv->p_config ;


	switch( cmd ) {
		case ICX_KEY_IOCTL_CMD_GETCOUNT:
		{
			return p_config->count ;
		}

		case ICX_KEY_IOCTL_CMD_GETINFO:
		{
			struct cy8cmbr31xx_keyinfo	const *p_key_info ;
			u32							buff[BUTTON_MAX] ;
			uint						index ;


			p_key_info = &p_config->p_key_info[0] ;
			for(index=0; index<p_config->count; index++,p_key_info++) {
				buff[index] = p_key_info->key_code ;
			}

			if( copy_to_user((void __user *)arg, &buff[0], sizeof( u32 )*p_config->count) ) {
				return -EFAULT ;
			}
			break ;
		}

		default:
			return -EINVAL ;
	}

	return 0 ;
}


static const struct file_operations		cy8cmbr31xx_fops={
											.owner          = THIS_MODULE,
											.open           = cy8cmbr31xx_open,
											.unlocked_ioctl = cy8cmbr31xx_ioctl,
											.compat_ioctl   = cy8cmbr31xx_ioctl,
										} ;


/* -------------------------------------------------------------------------------- */
/*   プローブ                                                                       */
/*                                                                                  */
/*   IN : p_client = I2C クライアント                                               */
/*        p_id     = I2C デバイスの識別子                                           */
/*   OUT: 0       == 正常終了                                                       */
/*        0       <  異常終了                                                       */
/* -------------------------------------------------------------------------------- */
static int cy8cmbr31xx_probe(struct i2c_client *p_client, const struct i2c_device_id *p_id)
{
#ifdef  DEBUG_CONFIG
	struct cy8cmbr31xx_config		*p_config=&cy8cmbr3108_config ;
#else
	struct cy8cmbr31xx_config		*p_config=p_client->dev.platform_data ;
#endif/*DEBUG_CONFIG*/
	struct cy8cmbr31xx_keyinfo		const *p_key_info ;
	struct priv						*p_priv ;
	int								result ;
	uint							index ;

	/* プライベートデータ確保 */
	p_priv = kzalloc(sizeof( struct priv ), GFP_KERNEL) ;
	if( p_priv == NULL ) {
		ERRTRACE("%s(%d)", __func__, __LINE__) ;
		return -ENOMEM ;
	}

	/* 初期化 */
	i2c_set_clientdata(p_client, p_priv) ;
	p_priv->p_client  = p_client ;
	p_priv->p_config  = p_config ;
	p_priv->type      = (int )p_id->driver_data ;
	p_priv->state     = 0 ;

	/* MISC デバイス作成 */
	p_priv->misc.minor = MISC_DYNAMIC_MINOR ;
	p_priv->misc.name  = DRV_NAME ;
	p_priv->misc.fops  = &cy8cmbr31xx_fops ;
	result             = misc_register( &p_priv->misc ) ;
	if( result ) {
		ERRTRACE("%s(%d)", __func__, __LINE__) ;
		result = -ENOMEM ;
		goto cy8cmbr31xx_module_init_f00 ;
	}

	/* 入力デバイス確保 */
	p_priv->p_input = input_allocate_device() ;
	if( p_priv->p_input == NULL ) {
		ERRTRACE("%s(%d)", __func__, __LINE__) ;
		result = -ENOMEM ;
		goto cy8cmbr31xx_module_init_e00 ;
	}

	/* 入力デバイス登録 */
	p_key_info = &p_config->p_key_info[0] ;
	for(index=0; index<p_config->count; index++,p_key_info++) {
		__set_bit(p_key_info->key_code, p_priv->p_input->keybit) ;
		p_priv->mask |= p_key_info->key_bit ;
	}
	__set_bit(EV_KEY, p_priv->p_input->evbit) ;
	p_priv->p_input->name       = DRV_NAME ;
	p_priv->p_input->phys       = DRV_NAME ;
	p_priv->p_input->id.bustype = BUS_HOST ;
	p_priv->p_input->id.vendor  = 0x0001 ;
	p_priv->p_input->id.product = 0x0001 ;
	p_priv->p_input->id.version = 0x0100 ;
	p_priv->p_input->dev.parent = &p_client->dev ;
	result                      = input_register_device( p_priv->p_input ) ;
	if( result ) {
		input_free_device( p_priv->p_input ) ;
		ERRTRACE("%s(%d)", __func__, __LINE__) ;
		goto cy8cmbr31xx_module_init_e00 ;
	}

	/* ワークキュー作成 */
	p_priv->p_workqueue = create_workqueue( DRV_NAME ) ;
	if( p_priv->p_workqueue == NULL ) {
		ERRTRACE("%s(%d)", __func__, __LINE__) ;
		result = -ENOMEM ;
		goto cy8cmbr31xx_module_init_d00 ;
	}
	INIT_WORK(&p_priv->work, (void *)workqueue_proc) ;

	m_p_priv = p_priv ;

	/* フラッシュプログラム */
	result = programFlash(p_priv, &p_config->p_regs[0]) ;
	if( result ) {
		result = programFlash(p_priv, &p_config->p_regs[0]) ;
	}
	if( !result ) {
		p_priv->init = 1 ;

		if( p_config->eint_poll >= 0 ) {	/* 割り込み */
			mt_eint_registration(p_config->eint_poll, EINTF_TRIGGER_RISING, host_isr, 1) ;
			queue_work(p_priv->p_workqueue, &p_priv->work) ;
		} else {							/* ポーリング */
			init_timer( &p_priv->timer ) ;
			p_priv->polling        = msecs_to_jiffies( -p_config->eint_poll ) ;
			p_priv->timer.expires  = jiffies + p_priv->polling ;
			p_priv->timer.data     = (unsigned long )p_priv ;
			p_priv->timer.function = timer_proc ;
			add_timer( &p_priv->timer ) ;
		}
	}

	return 0 ;


cy8cmbr31xx_module_init_d00:
	input_unregister_device( p_priv->p_input ) ;
cy8cmbr31xx_module_init_e00:
	misc_deregister( &p_priv->misc ) ;
cy8cmbr31xx_module_init_f00:
	kfree( p_priv ) ;

	return result ;
}


/* -------------------------------------------------------------------------------- */
/*   削除                                                                           */
/*                                                                                  */
/*   IN : p_client = I2C クライアント                                               */
/*   OUT: 常に０                                                                    */
/* -------------------------------------------------------------------------------- */
static int cy8cmbr31xx_remove( struct i2c_client *p_client )
{
	struct priv					*p_priv=i2c_get_clientdata( p_client ) ;
	struct cy8cmbr31xx_config	*p_config=p_priv->p_config ;


	if( p_priv->init ) {
		if( p_config->eint_poll >= 0 ) {	/* 割り込み */
			mt_eint_registration(p_config->eint_poll, EINTF_TRIGGER_RISING, NULL, 1) ;
		} else {							/* ポーリング */
			del_timer( &p_priv->timer ) ;
		}
	}

	destroy_workqueue( p_priv->p_workqueue ) ;

	input_unregister_device( p_priv->p_input ) ;

	misc_deregister( &p_priv->misc ) ;

	kfree( p_priv ) ;

	return 0 ;
}


/* -------------------------------------------------------------------------------- */
/*   サスペンド処理                                                                 */
/*                                                                                  */
/*   IN : p_client = I2C クライアント                                               */
/*        message  = メッセージ                                                     */
/*   OUT: 常に０                                                                    */
/* -------------------------------------------------------------------------------- */
static int cy8cmbr31xx_suspend(struct i2c_client *p_client, pm_message_t mesg)
{
	struct priv					*p_priv=i2c_get_clientdata( p_client ) ;
	struct cy8cmbr31xx_config	*p_config=p_priv->p_config ;
	int							result ;


	if( !p_priv->init ) {
		return 0 ;
	}

	cancel_work_sync( &p_priv->work ) ;

	if( p_config->eint_poll >= 0 ) {	/* 割り込み */
		DBGTRACE("suspend - interrupt") ;
		mt_eint_mask(p_config->eint_poll);
		mt_eint_registration(p_config->eint_poll, EINTF_TRIGGER_RISING, NULL, 0) ;
		mt_eint_mask(p_config->eint_poll);
	} else {							/* ポーリング */
		DBGTRACE("suspend - polling") ;
		del_timer( &p_priv->timer ) ;
	}

	/* 省電力モード */
	result = write_reg8(p_priv, CTRL_CMD, POWER_SAVE_MODE, 0) ;
	if( !result ) {
		DBGTRACE("power save mode = OK") ;
	} else {
		ERRTRACE("power save mode = NG") ;
	}

	return 0 ;
}


/* -------------------------------------------------------------------------------- */
/*   レジューム処理                                                                 */
/*                                                                                  */
/*   IN : p_client = I2C クライアント                                               */
/*   OUT: 0                                                                         */
/* -------------------------------------------------------------------------------- */
static int cy8cmbr31xx_resume( struct i2c_client *p_client )
{
	struct priv					*p_priv=i2c_get_clientdata( p_client ) ;
	struct cy8cmbr31xx_config	*p_config=p_priv->p_config ;


	if( !p_priv->init ) {
		return 0 ;
	}

	if( p_config->eint_poll >= 0 ) {	/* 割り込み */
		DBGTRACE("resume - interrupt") ;
		mt_eint_registration(p_config->eint_poll, EINTF_TRIGGER_RISING, host_isr, 1) ;
		queue_work(p_priv->p_workqueue, &p_priv->work) ;
	} else {							/* ポーリング */
		DBGTRACE("resume - polling") ;
		p_priv->timer.expires  = jiffies + p_priv->polling ;
		add_timer( &p_priv->timer ) ;
	}

	return 0 ;
}


static const struct i2c_device_id	cy8cmbr31xx_id[] = {
										{ "cy8cmbr3102",  TYPE_CY8CMBR3102  },
										{ "cy8cmbr3106s", TYPE_CY8CMBR3106S },
										{ "cy8cmbr3108",  TYPE_CY8CMBR3108  },
										{ "cy8cmbr3110",  TYPE_CY8CMBR3110  },
										{ "cy8cmbr3116",  TYPE_CY8CMBR3116  },
										{}
									} ;

static struct i2c_driver			cy8cmbr31xx_driver = {
										.probe     = cy8cmbr31xx_probe,
										.remove    = cy8cmbr31xx_remove,
										.suspend   = cy8cmbr31xx_suspend,
										.resume    = cy8cmbr31xx_resume,
										.id_table  = cy8cmbr31xx_id,
										.driver    = {
											.name  = DRV_NAME,
											.owner = THIS_MODULE,
										},
									} ;


module_i2c_driver( cy8cmbr31xx_driver ) ;


MODULE_DESCRIPTION("Sony CY8CMBR31xx Driver");
MODULE_AUTHOR("Sony Corporation");
MODULE_LICENSE("GPL") ;
MODULE_ALIAS("Sony CY8CMBR31xx Driver") ;
MODULE_VERSION( "v0.01" ) ;
