
/* regs.h: Sony CY8CMBR31xx Driver.
 *
 * Copyright 2016 Sony Corporation.
 * Author: Sony Corporation.
 *
 * This package is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef __CY8CMBR31XX__REGS_H__INCLUDED__
#define __CY8CMBR31XX__REGS_H__INCLUDED__

	/* レジスタアドレス */
	enum {
		DATA_OFFSET					=	0x00,
		SENSOR_EN					=	DATA_OFFSET,
		FSS_EN						=	0x02,
		TOGGLE_EN					=	0x04,
		LED_ON_EN					=	0x06,
		SENSITIVITY0				=	0x08,
		SENSITIVITY1				=	0x09,
		SENSITIVITY2				=	0x0a,
		SENSITIVITY3				=	0x0b,
		BASE_THRESHOLD0 			=	0x0c,
		BASE_THRESHOLD1 			=	0x0d,
		FINGER_THRESHOLD2			=	0x0e,
		FINGER_THRESHOLD3			=	0x0f,
		FINGER_THRESHOLD4			=	0x10,
		FINGER_THRESHOLD5			=	0x11,
		FINGER_THRESHOLD6			=	0x12,
		FINGER_THRESHOLD7			=	0x13,
		FINGER_THRESHOLD8			=	0x14,
		FINGER_THRESHOLD9			=	0x15,
		FINGER_THRESHOLD10			=	0x16,
		FINGER_THRESHOLD11			=	0x17,
		FINGER_THRESHOLD12			=	0x18,
		FINGER_THRESHOLD13			=	0x19,
		FINGER_THRESHOLD14			=	0x1a,
		FINGER_THRESHOLD15			=	0x1b,
		SENSOR_DEBOUNCE 			=	0x1c,
		BUTTON_HYS					=	0x1d,
		BUTTON_LBR					=	0x1f,
		BUTTON_NNT					=	0x20,
		BUTTON_NT					=	0x21,
		PROX_EN 					=	0x26,
		PROX_CFG					=	0x27,
		PROX_CFG2					=	0x28,
		PROX_TOUCH_TH0				=	0x2a,
		PROX_TOUCH_TH1				=	0x2c,
		PROX_RESOLUTION0			=	0x2e,
		PROX_RESOLUTION1			=	0x2f,
		PROX_HYS					=	0x30,
		PROX_LBR					=	0x32,
		PROX_NNT					=	0x33,
		PROX_NT 					=	0x34,
		PROX_POSITIVE_TH0			=	0x35,
		PROX_POSITIVE_TH1			=	0x36,
		PROX_NEGATIVE_TH0			=	0x39,
		PROX_NEGATIVE_TH1			=	0x3a,
		LED_ON_TIME					=	0x3d,
		BUZZER_CFG					=	0x3e,
		BUZZER_ON_TIME				=	0x3f,
		GPO_CFG						=	0x40,
		PWM_DUTYCYCLE_CFG0			=	0x41,
		PWM_DUTYCYCLE_CFG1			=	0x42,
		PWM_DUTYCYCLE_CFG2			=	0x43,
		PWM_DUTYCYCLE_CFG3			=	0x44,
		PWM_DUTYCYCLE_CFG4			=	0x45,
		PWM_DUTYCYCLE_CFG5			=	0x46,
		PWM_DUTYCYCLE_CFG6			=	0x47,
		PWM_DUTYCYCLE_CFG7			=	0x48,
		SPO_CFG 					=	0x4c,
		DEVICE_CFG0 				=	0x4d,
		DEVICE_CFG1 				=	0x4e,
		DEVICE_CFG2 				=	0x4f,
		DEVICE_CFG3 				=	0x50,
		I2C_ADDR					=	0x51,
		REFRESH_CTRL				=	0x52,
		STATE_TIMEOUT				=	0x55,
		SLIDER_CFG					=	0x5d,
		SLIDER1_CFG 				=	0x61,
		SLIDER1_RESOLUTION			=	0x62,
		SLIDER1_THRESHOLD			=	0x63,
		SLIDER2_CFG 				=	0x67,
		SLIDER2_RESOLUTION			=	0x68,
		SLIDER2_THRESHOLD			=	0x69,
		SLIDER_LBR					=	0x71,
		SLIDER_NNT					=	0x72,
		SLIDER_NT					=	0x73,
		CONFIG_CRC					=	0x7e,
		GPO_OUTPUT_STATE			=	0x80,
		SENSOR_ID					=	0x82,
		CTRL_CMD					=	0x86,
			CMD_OP_CODE_0			=	0,
			SAVE_CHECK_CRC			=	2,
			CMD_OP_CODE_3			=	3,
			POWER_SAVE_MODE			=	7,
			CMD_OP_CODE_8			=	8,
			CMD_OP_CODE_9			=	9,
			CMD_OP_CODE_10			=	10,
			SW_RESET				=	255,
		CTRL_CMD_STATUS 			=	0x88,
		CTRL_CMD_ERR				=	0x89,
		SYSTEM_STATUS				=	0x8a,
		PREV_CTRL_CMD_CODE			=	0x8c,
		FAMILY_ID					=	0x8f,
		DEVICE_ID					=	0x90,
		DEVICE_REV					=	0x92,
		CALC_CRC					=	0x94,
		TOTAL_WORKING_SNS			=	0x97,
		SNS_CP_HIGH 				=	0x98,
		SNS_VDD_SHORT				=	0x9a,
		SNS_GND_SHORT				=	0x9c,
		SNS_SNS_SHORT				=	0x9e,
		CMOD_SHIELD_TEST			=	0xa0,
		BUTTON_STAT 				=	0xaa,
		LATCHED_BUTTON_STAT 		=	0xac,
		PROX_STAT					=	0xae,
		LATCHED_PROX_STAT			=	0xaf,
		SLIDER1_POSITION			=	0xb0,
		LIFTOFF_SLIDER1_POSITION	=	0xb1,
		SLIDER2_POSITION			=	0xb2,
		LIFTOFF_SLIDER2_POSITION	=	0xb3,
		SYNC_COUNTER0				=	0xb9,
		DIFFERENCE_COUNT_SENSOR0	=	0xba,
		DIFFERENCE_COUNT_SENSOR1	=	0xbc,
		DIFFERENCE_COUNT_SENSOR2	=	0xbe,
		DIFFERENCE_COUNT_SENSOR3	=	0xc0,
		DIFFERENCE_COUNT_SENSOR4	=	0xc2,
		DIFFERENCE_COUNT_SENSOR5	=	0xc4,
		DIFFERENCE_COUNT_SENSOR6	=	0xc6,
		DIFFERENCE_COUNT_SENSOR7	=	0xc8,
		DIFFERENCE_COUNT_SENSOR8	=	0xca,
		DIFFERENCE_COUNT_SENSOR9	=	0xcc,
		DIFFERENCE_COUNT_SENSOR10	=	0xce,
		DIFFERENCE_COUNT_SENSOR11	=	0xd0,
		DIFFERENCE_COUNT_SENSOR12	=	0xd2,
		DIFFERENCE_COUNT_SENSOR13	=	0xd4,
		DIFFERENCE_COUNT_SENSOR14	=	0xd6,
		DIFFERENCE_COUNT_SENSOR15	=	0xd8,
		GPO_DATA					=	0xda,
		SYNC_COUNTER1				=	0xdb,
		DEBUG_SENSOR_ID 			=	0xdc,
		DEBUG_CP					=	0xdd,
		DEBUG_DIFFERENCE_COUNT0 	=	0xde,
		DEBUG_BASELINE0 			=	0xe0,
		DEBUG_RAW_COUNT0			=	0xe2,
		DEBUG_AVG_RAW_COUNT0		=	0xe4,
		SYNC_COUNTER2				=	0xe7
	} ;


#endif/*__CY8CMBR31XX__REGS_H__INCLUDED__*/
